#!/usr/bin/env ruby
###########################################################################
###########################################################################
##  Copyright The Resistance (C) 1995-2011                               ##
##  Author: blass                                                        ##
##  blass[at]sdf.lonestar.org                                            ##
##  Badass 0.2 Beta                                                      ##
##  This file is part of Badass.                                         ##
##  Badass is free software: you can redistribute it and/or modify       ##
##  it under the terms of the GNU General Public License as published by ##
##  the Free Software Foundation, either version 3 of the License, or    ##
##  (at your option) any later version.                                  ##
##  Badass is distributed in the hope that it will be useful,            ##
##  but WITHOUT ANY WARRANTY; without even the implied warranty of       ##
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ##
##  GNU General Public License for more details.                         ##
##  You should have received a copy of the GNU General Public License    ##
##  along with Badass. If not, see http://www.gnu.org/licenses/          ##
###########################################################################
###########################################################################

module Racket
  module L2
    # VLAN Trunking Protocol (VTP)
    # http://en.wikipedia.org/wiki/VLAN_Trunking_Protocol
    # http://www.cisco.com/en/US/tech/tk389/tk689/technologies_tech_note09186a0080094c52.shtml
    # This is just a base class from which all VTP messages inherit and should never be used directly
    class VTPGeneric < RacketPart
      # VTP version (1-3)
      unsigned :version, 8 
      # Message code (summary advertisement, subset advertisement, advertisement request, VTP join)
      unsigned :code, 8 
      # Sometimes used, sometimes not, depends on the type
  unsigned :reserved, 8  
      # Length of the management domain
      unsigned :domain_length, 8
      # management domain name, zero padded to 32 bytes
      text :domain, 256
      
      # Adjust +domain_length+ and +domain+ accordingly prior to sending
      def fix!
        self.domain_length = self.domain.length 
        self.domain = self.domain.ljust(32, "\x00")
      end
    end
    
    # A raw VTP message
    class VTPRaw < VTPGeneric
      rest :payload
    end
    
    class VTPSubsetAdvertisement < VTPGeneric
      # Configuration revision
      unsigned :revision, 32
      # all of the vlan info fields
      rest :payload
      
      def add_vlan_info(status, type, id, mtu, index, name)
        name_length = name.length
        # zero pad name to a multiple of 4 bytes
        name = name.length % 4 == 0 ? name : name.ljust(name.length + (4 - (name.length % 4)), "\x00")
        length = 12 + name.length
        @vlan_info << [length, status, type, name_length, id, mtu, index, name]
      end
      
      def fix!
        @vlan_info.each do |v|
          self.payload += v.pack("CCCCnnNa*")
        end
        super
      end
      
      def initialize(*args)
        @vlan_info = []
        super(*args)
        self.code = 2 
      end
    end
    
    class VTPSummaryAdvertisement < VTPGeneric
      # Configuration revision number
      unsigned :revision, 32
      # Updater identity (IP)
      octets :updater, 32
      # update timestamp
      unsigned :timestamp, 96
      # MD5 digest of VTP password
      text :md5, 128 
      rest :payload
      
      def initialize(*args)
        super(*args)
        self.code = 1
      end
    end
    
    class VTPAdvertisementRequest < VTPGeneric
      # This is used in cases in which there are several subset advertisements. If the first (n) subset advertisement has been received and the subsequent one (n+1) has not been received, the Catalyst only requests advertisements from the (n+1)th one.
      unsigned :start, 32
      rest :payload
      def initialize(*args)
        super(*args)
        self.code = 3
      end
      
    end
    
    
    class VTPJoin < VTPGeneric
      def initialize(*args)
        super(*args)
        self.code = 4
      end
    end
    
  end
end
# vim: set ts=2 et sw=2:
